import { useEffect, useRef, useState } from 'react'
import 'ol/ol.css'
import { Modify } from 'ol/interaction'
import { ImageStatic, Vector as VectorSource } from 'ol/source'
import Feature from 'ol/Feature'
import { IRectCoords, SatelliteMapsProvider } from '../../interfaces/map'
import { Extent, getCenter } from 'ol/extent'
import { highlightStyleRed, highlightStyleYellow } from './MapStyles'
import { customMapSource, googleMapsSatelliteSource, yandexMapsSatelliteSource } from './MapSources'
import { Geometry, LineString, SimpleGeometry } from 'ol/geom'
import { fromExtent } from 'ol/geom/Polygon'
import { Coordinate } from 'ol/coordinate'
import { addInteractions, handleImageDrop, loadFeatures, processFigure, processLine } from './mapUtils'
import useSWR, { SWRConfiguration } from 'swr'
import { fetcher } from '../../http/axiosInstance'
import { BASE_URL } from '../../constants'
import { ActionIcon, Autocomplete, CloseButton, Flex, Select as MantineSelect, MantineStyleProp, rem, useMantineColorScheme, Portal, Menu, Button, Group, Divider, LoadingOverlay, Stack, Container, Modal, Transition } from '@mantine/core'
import { IconBoxMultiple, IconBoxPadding, IconChevronDown, IconPlus, IconSearch, IconUpload } from '@tabler/icons-react'
import { ICitySettings, IFigure, ILine } from '../../interfaces/gis'
import axios from 'axios'
import MapToolbar from './MapToolbar/MapToolbar'
import MapStatusbar from './MapStatusbar/MapStatusbar'
import { setAlignMode, setSatMapsProvider, setTypeRoles, useMapStore, setMapLabel, clearPrintArea } from '../../store/map'
import { useThrottle } from '@uidotdev/usehooks'
import ObjectTree from '../Tree/ObjectTree'
import { setCurrentObjectId, setSelectedDistrict, setSelectedRegion, setSelectedYear, useObjectsStore } from '../../store/objects'
import MapLayers from './MapLayers/MapLayers'
import ObjectParameters from './ObjectParameters/ObjectParameters'
import TabsPane, { ITabsPane } from './TabsPane/TabsPane'
//import { useSearchParams } from 'react-router-dom'
import GeoJSON from 'ol/format/GeoJSON'
import MapLegend from './MapLegend/MapLegend'
import GisService from '../../services/GisService'
import MapMode from './MapMode'

const satMapsProviders = [
    { label: 'Google', value: 'google' },
    { label: 'Яндекс', value: 'yandex' },
    { label: 'Подложка', value: 'custom' },
    { label: 'Static', value: 'static' }
]

const swrOptions: SWRConfiguration = {
    revalidateOnFocus: false
}

function getCitySettings() {
    return {
        city_id: 0,
        image_width: 8500,
        image_height: 12544,
        scale: 9000,
        offset_x: 0,//14442665.697619518,
        offset_y: 0,//8884520.63524492,
        rotation: 0
    } as ICitySettings
}

const MapComponent = ({
    id,
    active = false
}: {
    id: string,
    active: boolean,
}) => {
    // Store
    const { colorScheme } = useMantineColorScheme()
    const { selectedYear, currentObjectId, selectedRegion, selectedDistrict } = useObjectsStore().id[id]
    const {
        mode, map, currentTool, alignMode, satMapsProvider,
        selectedObjectType, file, measureDraw,
        polygonExtent, rectCoords, draw, snap, translate,
        nodeLayerSource, drawingLayerSource,
        satLayer, staticMapLayer, figuresLayer, linesLayer,
        regionsLayer, districtBoundLayer, baseLayer,
        printArea, printSource, printAreaDraw
    } = useMapStore().id[id]

    // Tab settings
    const objectsPane: ITabsPane[] = [{ title: 'Объекты', value: 'objects', view: <ObjectTree map_id={id} /> }, { title: 'Неразмещенные', value: 'unplaced', view: <></> }, { title: 'Другие', value: 'other', view: <></> },]
    const paramsPane: ITabsPane[] = [{ title: 'История изменений', value: 'history', view: <></> }, { title: 'Параметры', value: 'parameters', view: <ObjectParameters map_id={id} /> }, { title: 'Вычисляемые', value: 'calculated', view: <></> }]

    // Map
    const mapElement = useRef<HTMLDivElement | null>(null)
    const tooltipRef = useRef<HTMLDivElement | null>(null)

    useSWR(`/gis/type-roles`, (url) => fetcher(url, BASE_URL.ems).then(res => {
        if (Array.isArray(res)) {
            setTypeRoles(id, res)
        }
        return res
    }), swrOptions)

    const { data: regionBoundsData } = useSWR(`/gis/bounds/region`, (url) => fetcher(url, BASE_URL.ems), swrOptions)

    useEffect(() => {
        if (regionsLayer && regionBoundsData) {
            if (Array.isArray(regionBoundsData)) {
                regionBoundsData.map(bound => {
                    const geoJson = new GeoJSON({ featureProjection: 'EPSG:3857' })
                    const geometry = geoJson.readGeometry(bound) as Geometry

                    const feature = new Feature(geometry)
                    feature.setProperties(bound.properties)

                    regionsLayer.getSource()?.addFeature(feature)
                })
            }
            //regionsLayer.current.getSource()?.addFeature()
        }
    }, [regionBoundsData])

    useEffect(() => {
        if (selectedDistrict && selectedYear) {
            const bounds = new VectorSource({
                url: `${BASE_URL.ems}/gis/bounds/city/${selectedDistrict}`,
                format: new GeoJSON(),
            })

            districtBoundLayer.setSource(bounds)

            bounds.on('featuresloadend', function () {
                // map.current?.setView(new View({
                //     extent: bounds.getExtent()
                // }))
                console.log(bounds.getFeatures().length)
            })
        }
    }, [selectedDistrict, selectedYear])

    useEffect(() => {
        map?.setTarget(mapElement.current as HTMLDivElement)

        if (drawingLayerSource) {
            const modify = new Modify({ source: drawingLayerSource })
            map?.addInteraction(modify)
        }

        loadFeatures(id)
    }, [])

    useEffect(() => {
        if (currentTool) {
            if (draw) map?.removeInteraction(draw)
            //if (snap.current) map?.current?.removeInteraction(snap.current)
            addInteractions(id)
        } else {
            if (translate) map?.removeInteraction(translate)
            if (draw) map?.removeInteraction(draw)
            if (snap) map?.removeInteraction(snap)
            if (measureDraw) map?.removeInteraction(measureDraw)
        }
    }, [currentTool])

    // Satellite tiles setting
    useEffect(() => {
        if (satLayer) {
            if (satMapsProvider === 'google') {
                satLayer.setSource(googleMapsSatelliteSource)
            }

            if (satMapsProvider === 'yandex') {
                satLayer.setSource(yandexMapsSatelliteSource)
            }

            if (satMapsProvider === 'custom') {
                satLayer.setSource(customMapSource)
            }
        }
    }, [satMapsProvider, satLayer])

    const submitOverlay = async (file: File | null, polygonExtent: Extent | undefined, rectCoords: IRectCoords | undefined) => {
        await GisService.uploadOverlay(file, polygonExtent, rectCoords).then(res => {
            console.log(res)
        })
    }

    const mapControlsStyle: MantineStyleProp = {
        borderRadius: '4px',
        zIndex: '1',
        backgroundColor: colorScheme === 'light' ? '#F0F0F0CC' : '#000000CC',
        backdropFilter: 'blur(8px)',
        border: '1px solid #00000022'
    }

    const { data: nodes } = useSWR('/nodes/all', () => fetcher('/nodes/all', BASE_URL.ems), { revalidateOnFocus: false })

    useEffect(() => {
        // Draw features based on database data
        if (Array.isArray(nodes)) {
            nodes.map(node => {
                if (node.shape_type === 'LINE') {
                    const coordinates: Coordinate[] = []
                    if (Array.isArray(node.shape)) {
                        node.shape.map((point: { x: number, y: number }) => {
                            const coordinate = [point.x as number, point.y as number] as Coordinate
                            coordinates.push(coordinate)
                        })
                    }
                    //console.log(coordinates)
                    nodeLayerSource.addFeature(new Feature({ geometry: new LineString(coordinates) }))
                }
            })
        }
    }, [nodes, nodeLayerSource])

    const [searchObject, setSearchObject] = useState<string | undefined>("")
    const throttledSearchObject = useThrottle(searchObject, 500);

    useEffect(() => {
        if (!selectedObjectType || !map) return;

        if (figuresLayer) {
            // Reset styles and apply highlight to matching features
            figuresLayer.getSource()?.getFeatures().forEach((feature) => {
                if (selectedObjectType == feature.get('type')) {
                    feature.setStyle(highlightStyleYellow);
                } else {
                    feature.setStyle(undefined); // Reset to default style
                }
            })
        }

        if (linesLayer) {
            // Reset styles and apply highlight to matching features
            linesLayer.getSource()?.getFeatures().forEach((feature) => {
                if (selectedObjectType == feature.get('type')) {
                    feature.setStyle(highlightStyleYellow);
                } else {
                    feature.setStyle(undefined); // Reset to default style
                }
            })
        }
    }, [selectedObjectType])

    useEffect(() => {
        if (currentObjectId) {
            if (figuresLayer) {
                // Reset styles and apply highlight to matching features
                figuresLayer.getSource()?.getFeatures().forEach((feature: Feature) => {
                    if (currentObjectId == feature.get('object_id')) {
                        feature.setStyle(highlightStyleRed)

                        const geometry = feature.getGeometry()

                        if (geometry) {
                            map?.getView().fit(geometry as SimpleGeometry, { duration: 500, maxZoom: 18 })
                        }

                    } else {
                        feature.setStyle(undefined); // Reset to default style
                    }
                })
            }

            if (linesLayer) {
                // Reset styles and apply highlight to matching features
                linesLayer.getSource()?.getFeatures().forEach((feature: Feature) => {
                    if (currentObjectId == feature.get('object_id')) {
                        feature.setStyle(highlightStyleRed);

                        const geometry = feature.getGeometry()
                        if (geometry) {
                            map?.getView().fit(geometry as SimpleGeometry, { duration: 500, maxZoom: 18 })
                        }
                    } else {
                        feature.setStyle(undefined); // Reset to default style
                    }
                })
            }
        }
    }, [currentObjectId])

    const { data: regionsData } = useSWR(`/general/regions/all`, (url) => fetcher(url, BASE_URL.ems), swrOptions)

    const { data: districtsData } = useSWR(selectedRegion ? `/general/districts/all?region_id=${selectedRegion}` : null, (url) => fetcher(url, BASE_URL.ems), swrOptions)

    const { data: searchData } = useSWR(throttledSearchObject !== "" && selectedDistrict && selectedYear ? `/general/search/objects?q=${throttledSearchObject}&id_city=${selectedDistrict}&year=${selectedYear}` : null, (url) => fetcher(url, BASE_URL.ems), swrOptions)

    const { data: figuresData, isValidating: figuresValidating } = useSWR(
        selectedDistrict && selectedYear ? `/gis/figures/all?city_id=${selectedDistrict}&year=${selectedYear}&offset=0&limit=${10000}` : null,
        (url) => axios.get(url, {
            baseURL: BASE_URL.ems
        }).then((res) => res.data),
        swrOptions
    )

    const { data: linesData, isValidating: linesValidating } = useSWR(
        !figuresValidating && selectedDistrict && selectedYear ? `/gis/lines/all?city_id=${selectedDistrict}&year=${selectedYear}&offset=0&limit=${10000}` : null,
        (url) => axios.get(url, {
            baseURL: BASE_URL.ems
        }).then((res) => res.data),
        swrOptions
    )

    const { data: districtData } = useSWR(
        selectedDistrict ? `/gis/images/all?city_id=${selectedDistrict}` : null,
        (url) => axios.get(url, {
            baseURL: BASE_URL.ems
        }).then((res) => Array.isArray(res.data) ? res.data[0] : null),
        swrOptions
    )

    useEffect(() => {
        if (Array.isArray(districtsData)) {
            districtsData.map((district) => {
                if (district.id === selectedDistrict) {
                    setMapLabel(id, [district.name, district.district_name, selectedYear].join(' - '))
                }
            })
        }
    }, [districtsData, id, selectedDistrict, selectedYear])

    //const [searchParams, setSearchParams] = useSearchParams()

    // useEffect(() => {
    //     if (selectedRegion) {
    //         setSearchParams((params) => {
    //             params.set('r', selectedRegion.toString());
    //             return params
    //         })
    //     }

    //     if (selectedDistrict) {
    //         setSearchParams((params) => {
    //             params.set('d', selectedDistrict?.toString());
    //             return params
    //         })
    //     }

    //     if (selectedYear) {
    //         setSearchParams((params) => {
    //             params.set('y', selectedYear?.toString());
    //             return params
    //         })
    //     }

    //     if (currentObjectId) {
    //         setSearchParams((params) => {
    //             params.set('o', currentObjectId?.toString());
    //             return params
    //         })
    //     }
    // }, [selectedRegion, selectedDistrict, selectedYear, currentObjectId, setSearchParams])

    // useEffect(() => {
    //     if (Array.isArray(regionsData)) {
    //         const region = searchParams.get('r')

    //         if (searchParams.get('r')) {
    //             setSelectedRegion(Number(region))
    //         }
    //     }
    // }, [searchParams, regionsData])

    // useEffect(() => {
    //     if (Array.isArray(regionsData)) {
    //         const district = searchParams.get('d')

    //         if (Array.isArray(districtsData)) {
    //             if (district) {
    //                 setSelectedDistrict(Number(district))
    //             }
    //         }
    //     }
    // }, [searchParams, regionsData, districtsData])


    // useEffect(() => {
    //     if (Array.isArray(regionsData)) {
    //         const year = searchParams.get('y')

    //         if (year) {
    //             setSelectedYear(Number(year))
    //         }
    //     }
    // }, [searchParams, regionsData])

    // useEffect(() => {
    //     const object = searchParams.get('o')

    //     if (figuresData && linesData && object) {
    //         setCurrentObjectId(object)
    //     }
    // }, [searchParams, figuresData, linesData])

    useEffect(() => {
        if (selectedDistrict === null) {
            setSelectedYear(id, null)
        }

        if (selectedRegion === null) {
            setSelectedYear(id, null)
            setSelectedDistrict(id, null)
        }
    }, [selectedDistrict, selectedRegion, id])

    useEffect(() => {
        const districtBoundSource = districtBoundLayer.getSource()

        if (selectedDistrict && districtBoundSource && districtBoundSource.getFeatures().length > 0) {
            const center: Coordinate = getCenter(districtBoundSource.getExtent())
            const settings = getCitySettings()

            if (Array.isArray(figuresData)) {
                figuresLayer.getSource()?.clear()
                if (figuresData.length > 0) {
                    const geoJsonObject = {
                        type: "FeatureCollection",
                        features: figuresData.map((figure: IFigure) => processFigure(
                            figure,
                            settings.scale,
                            [center[0], center[1]]
                        )),
                    }

                    const features = new GeoJSON().readFeatures(geoJsonObject)

                    figuresLayer.getSource()?.addFeatures(features)

                    if (map) {
                        const extent = figuresLayer.getSource()?.getExtent()

                        if (extent) {
                            map.getView().fit(fromExtent(extent), { duration: 500 })
                        }
                    }
                }
            }

            if (Array.isArray(linesData)) {
                linesLayer.getSource()?.clear()
                if (linesData.length > 0) {
                    const geoJsonObject = {
                        type: "FeatureCollection",
                        features: linesData.map((line: ILine) => processLine(line, settings.scale, [center[0], center[1]])),
                    }

                    const features = new GeoJSON().readFeatures(geoJsonObject)

                    linesLayer.getSource()?.addFeatures(features)
                }
            }
        }

    }, [figuresData, linesData, selectedDistrict, selectedYear, districtBoundLayer])

    useEffect(() => {
        if (!selectedRegion) {
            setSelectedRegion(id, null)
            setSelectedYear(id, null)
        }
    }, [selectedRegion, selectedDistrict, id])

    useEffect(() => {
        if (selectedDistrict && districtData) {
            const settings = getCitySettings()

            const imageUrl = `${import.meta.env.VITE_API_EMS_URL}/static/${selectedDistrict}`;
            const img = new Image();
            img.src = imageUrl;
            img.onload = () => {
                if (map) {
                    const width = img.naturalWidth
                    const height = img.naturalHeight
                    //const k = (width < height ? width / height : height / width)
                    const k = settings.image_scale

                    const wk = width * k
                    const hk = height * k

                    const center = [settings.offset_x + (wk), settings.offset_y - (hk)];

                    const extent = [
                        center[0] - (wk),
                        center[1] - (hk),
                        center[0] + (wk),
                        center[1] + (hk),
                    ];

                    // Set up the initial image layer with the extent
                    const imageSource = new ImageStatic({
                        url: imageUrl,
                        imageExtent: extent,
                    });
                    staticMapLayer.setSource(imageSource);

                    //map.current.addLayer(imageLayer.current);
                }
            };
        }
    }, [selectedDistrict, districtData, staticMapLayer])

    useEffect(() => {
        if (baseLayer) {
            baseLayer.on('prerender', function (e) {
                if (colorScheme === 'dark') {
                    if (e.context) {
                        const context = e.context as CanvasRenderingContext2D;
                        context.filter = 'grayscale(80%) invert(100%) hue-rotate(180deg) ';
                        context.globalCompositeOperation = 'source-over';
                    }
                } else {
                    if (e.context) {
                        const context = e.context as CanvasRenderingContext2D;
                        context.filter = 'none';
                    }
                }
            })

            baseLayer.on('postrender', function (e) {
                if (e.context) {
                    const context = e.context as CanvasRenderingContext2D;
                    context.filter = 'none';
                }
            })
        }
    }, [colorScheme])

    useEffect(() => {
        if (map) {
            if (mode === 'print') {
                map.addInteraction(printAreaDraw)
            } else {
                map.removeInteraction(printAreaDraw)
            }
        }

    }, [mode, map, printAreaDraw])

    useEffect(() => {
        if (printArea) {
            map?.setTarget('print-portal')
            // map?.setView(new View({
            //     extent: printArea
            // }))
            printSource.clear()
            map?.getView().setCenter(getCenter(printArea))
            map?.getView().fit(printArea, {
                size: [640, 320]
            })
        }
    }, [printArea, map])

    return (
        <>
            <Modal keepMounted size='auto' opened={!!printArea} onClose={() => {
                clearPrintArea(id)
                map?.setTarget(mapElement.current as HTMLDivElement)
            }} title="Предпросмотр области">
                <div id='print-portal' style={{ width: '640px', height: '320px' }}>

                </div>
            </Modal>

            {active &&
                <Portal target='#header-portal'>
                    <Flex gap={'sm'} direction={'row'}>
                        <Autocomplete
                            form='search_object'
                            placeholder="Поиск"
                            flex={'1'}
                            data={searchData ? searchData.map((item: { value: string, id_object: string }) => ({ label: item.value, value: item.id_object.toString() })) : []}
                            //onSelect={(e) => console.log(e.currentTarget.value)}
                            onChange={(value) => setSearchObject(value)}
                            onOptionSubmit={(value) => setCurrentObjectId(id, value)}
                            rightSection={
                                searchObject !== '' && (
                                    <CloseButton
                                        size="sm"
                                        onMouseDown={(event) => event.preventDefault()}
                                        onClick={() => {
                                            setSearchObject('')
                                        }}
                                        aria-label="Clear value"
                                    />
                                )
                            }
                            leftSection={<IconSearch size={16} />}
                            value={searchObject}
                        />

                        <MantineSelect
                            placeholder="Регион"
                            flex={'1'}
                            data={regionsData ? regionsData.map((item: { name: string, id: number }) => ({ label: item.name, value: item.id.toString() })) : []}
                            onChange={(value) => setSelectedRegion(id, Number(value))}
                            clearable
                            onClear={() => {
                                setSelectedRegion(id, null)
                                // setSearchParams((params) => {
                                //     params.delete('r')
                                //     return params
                                // })
                            }}
                            searchable
                            value={selectedRegion ? selectedRegion.toString() : null}
                        />

                        <MantineSelect
                            placeholder="Населённый пункт"
                            flex={'1'}
                            data={districtsData ? districtsData.map((item: { name: string, id: number, district_name: string }) => ({ label: [item.name, item.district_name].join(' - '), value: item.id.toString() })) : []}
                            onChange={(value) => setSelectedDistrict(id, Number(value))}
                            clearable
                            onClear={() => {
                                setSelectedDistrict(id, null)
                                // setSearchParams((params) => {
                                //     params.delete('d')
                                //     return params
                                // })
                            }}
                            searchable
                            value={selectedDistrict ? selectedDistrict.toString() : null}
                        />

                        <MantineSelect placeholder='Схема' w='92px'
                            data={['2018', '2019', '2020', '2021', '2022', '2023', '2024'].map(el => ({ label: el, value: el }))}
                            onChange={(e) => {
                                if (e) {
                                    setSelectedYear(id, Number(e))
                                } else {
                                    setSelectedYear(id, null)
                                }
                            }}
                            onClear={() => {
                                setSelectedYear(id, null)
                                // setSearchParams((params) => {
                                //     params.delete('y')
                                //     return params
                                // })
                            }}
                            value={selectedYear ? selectedYear?.toString() : null}
                            clearable
                        />

                        <Button variant={alignMode ? 'filled' : 'transparent'} onClick={() => setAlignMode(id, !alignMode)}>
                            <IconBoxPadding style={{ width: rem(20), height: rem(20) }} />
                        </Button>

                        <Menu position="bottom-end" transitionProps={{ transition: 'pop-top-right' }}>
                            <Menu.Target>
                                <Button variant='transparent'>
                                    <Group gap={7} wrap='nowrap' style={{ flexShrink: 0 }} title='Слои'>
                                        <IconBoxMultiple style={{ width: rem(20), height: rem(20) }} />
                                        <IconChevronDown style={{ width: rem(12), height: rem(12) }} stroke={1.5} />
                                    </Group>
                                </Button>
                            </Menu.Target>
                            <Menu.Dropdown miw={300}>
                                <Menu.Label>{'Настройка видимости слоёв'}</Menu.Label>

                                <Flex p='sm' direction='column' gap='xs'>
                                    <Flex align='center' direction='row' gap='sm'>
                                        <MantineSelect value={satMapsProvider} data={satMapsProviders} onChange={(value) => setSatMapsProvider(id, value as SatelliteMapsProvider)} />
                                    </Flex>
                                    <Flex direction='row'>
                                        <ActionIcon size='lg' variant='transparent' onClick={() => submitOverlay(file, polygonExtent, rectCoords)}>
                                            <IconUpload style={{ width: rem(20), height: rem(20) }} />
                                        </ActionIcon>

                                        <ActionIcon size='lg' variant='transparent' title='Добавить подложку'>
                                            <IconPlus style={{ width: rem(20), height: rem(20) }} />
                                        </ActionIcon>
                                    </Flex>
                                    <MapLayers map={map} />
                                </Flex>
                            </Menu.Dropdown>
                        </Menu>
                    </Flex>
                </Portal>
            }

            <Container pos='absolute' w='100%' h='100%' p='0' fluid>
                <Flex direction='column' w='100%' h='100%'>
                    <Flex w='100%' h='94%' p='xs' style={{ flexGrow: 1 }}>
                        <Stack w='100%' maw='340px'>
                            {selectedRegion && selectedDistrict && selectedYear &&
                                <Flex direction='column' h={'100%'} w={'100%'} style={mapControlsStyle}>
                                    <TabsPane defaultTab='objects' tabs={objectsPane} />
                                    <Divider />
                                    <TabsPane defaultTab='parameters' tabs={paramsPane} />
                                </Flex>
                            }

                            <Transition
                                mounted={!!selectedRegion && !!selectedDistrict && !!selectedYear}
                                transition="slide-right"
                                duration={200}
                                timingFunction="ease"
                            >
                                {(styles) => <Flex direction='column' h={'100%'} w={'100%'} style={{ ...mapControlsStyle, ...styles }}>
                                    <TabsPane defaultTab='objects' tabs={objectsPane} />
                                    <Divider />
                                    <TabsPane defaultTab='parameters' tabs={paramsPane} />
                                </Flex>}
                            </Transition>
                        </Stack>

                        <Stack w='100%' align='center'>
                            <Stack style={mapControlsStyle} w='fit-content'>
                                <MapMode map_id={id} />
                            </Stack>
                        </Stack>

                        <Stack w='100%' maw='340px' align='flex-end' justify='space-between'>
                            {selectedRegion && selectedDistrict && selectedYear && mode === 'edit' &&
                                <MapToolbar map_id={id} />
                            }

                            <Transition
                                mounted={!!selectedRegion && !!selectedDistrict && !!selectedYear}
                                transition="slide-left"
                                duration={200}
                                timingFunction="ease"
                            >
                                {(styles) => <MapLegend style={styles} selectedDistrict={selectedDistrict} selectedYear={selectedYear} />}
                            </Transition>
                        </Stack>
                    </Flex>

                    <Flex w='100%'>
                        <MapStatusbar
                            map_id={id}
                            mapControlsStyle={mapControlsStyle}
                        />
                    </Flex>
                </Flex>
            </Container>

            <Container pos='absolute' fluid p={0} w='100%' h='100%' mah='100%' ref={mapElement} onDragOver={(e) => e.preventDefault()} onDrop={(e) => handleImageDrop(e, id)}>
                <div ref={tooltipRef}></div>
            </Container>

            <LoadingOverlay visible={linesValidating || figuresValidating} />
        </>
    )
};

export default MapComponent
